==> khipu_metabolomics-2.0.0-py3-none-any.whl: khipu/isotopes.py <==
# this contains the raw isotope data for the isotope correction portion of khipu

from decimal import Decimal
import pandas as pd
import numpy as np

# this is a master list of isotopes sourced from NIST
# update this if the NAPs in yours system are non-canonical
# see the function below on how to override this list.

ISOTOPE_RAW_DATA = """
    element,mass,abundance
    H,1.00782503223,0.999885
    H,2.01410177812,0.000115
    He,3.0160293201,1.34e-06
    He,4.00260325413,0.99999866
    Li,6.0151228874,0.0759
    Li,7.0160034366,0.9241
    Be,9.012183065,1.0
    B,10.01293695,0.199
    B,11.00930536,0.801
    C,12.0,0.9893
    C,13.00335483507,0.0107
    N,14.00307400443,0.99636
    N,15.00010889888,0.00364
    O,15.99491461957,0.99757
    O,16.9991317565,0.00038
    O,17.99915961286,0.00205
    F,18.99840316273,1.0
    Ne,19.9924401762,0.9048
    Ne,20.993846685,0.0027
    Ne,21.991385114,0.0925
    Na,22.989769282,1.0
    Mg,23.985041697,0.7899
    Mg,24.985836976,0.1
    Mg,25.982592968,0.1101
    Al,26.98153853,1.0
    Si,27.97692653465,0.92223
    Si,28.9764946649,0.04685
    Si,29.973770136,0.03092
    P,30.97376199842,1.0
    S,31.9720711744,0.9499
    S,32.9714589098,0.0075
    S,33.967867004,0.0425
    S,35.0000,0.0000
    S,35.96708071,0.0001
    Cl,34.968852682,0.7576
    Cl,36.0000,0.0000
    Cl,36.965902602,0.2424
    Ar,35.967545105,0.003336
    Ar,37.00000,0.0000
    Ar,37.96273211,0.000629
    Ar,39.00000,0.0000
    Ar,39.9623831237,0.996035
    K,38.9637064864,0.932581
    K,39.963998166,0.000117
    K,40.9618252579,0.067302
    Ca,39.962590863,0.96941
    Ca,41.0000,0.0000
    Ca,41.95861783,0.00647
    Ca,42.95876644,0.00135
    Ca,43.95548156,0.02086
    Ca,45.0000,0.0000
    Ca,45.953689,4e-05
    Ca,46.9000,0.0000
    Ca,47.95252276,0.00187
    Sc,44.95590828,1.0
    Ti,45.95262772,0.0825
    Ti,46.95175879,0.0744
    Ti,47.94794198,0.7372
    Ti,48.94786568,0.0541
    Ti,49.94478689,0.0518
    V,49.94715601,0.0025
    V,50.94395704,0.9975
    Cr,49.94604183,0.04345
    Cr,51.0000,0.0000
    Cr,51.94050623,0.83789
    Cr,52.94064815,0.09501
    Cr,53.93887916,0.02365
    Mn,54.93804391,1.0
    Fe,53.93960899,0.05845
    Fe,55.0000,0.0000
    Fe,55.93493633,0.91754
    Fe,56.93539284,0.02119
    Fe,57.93327443,0.00282
    Co,58.93319429,1.0
    Ni,57.93534241,0.68077
    Ni,59.0000,0.0000
    Ni,59.93078588,0.26223
    Ni,60.93105557,0.011399
    Ni,61.92834537,0.036346
    Ni,63.0000,0.0000
    Ni,63.92796682,0.009255
    Cu,62.92959772,0.6915
    Cu,64.0000,0.0000
    Cu,64.9277897,0.3085
    Zn,63.92914201,0.4917
    Zn,65.0000,0.0000
    Zn,65.92603381,0.2773
    Zn,66.92712775,0.0404
    Zn,67.92484455,0.1845
    Zn,69.0000,0.0000
    Zn,69.9253192,0.0061
    Ga,68.9255735,0.60108
    Ga,70.0000,0.0000
    Ga,70.92470258,0.39892
    Ge,69.92424875,0.2057
    Ge,71.0000,0.0000
    Ge,71.922075826,0.2745
    Ge,72.923458956,0.0775
    Ge,73.921177761,0.365
    Ge,75.0000,0.0000
    Ge,75.921402726,0.0773
    As,74.92159457,1.0
    Se,73.922475934,0.0089
    Se,75.0000,0.0000
    Se,75.919213704,0.0937
    Se,76.919914154,0.0763
    Se,77.91730928,0.2377
    Se,79.0000,0.0000
    Se,79.9165218,0.4961
    Se,81.0000,0.0000
    Se,81.9166995,0.0873
    Br,78.9183376,0.5069
    Br,80.0000,0.0000
    Br,80.9162897,0.4931
    Kr,77.92036494,0.00355
    Kr,79.0000,0.0000
    Kr,79.91637808,0.02286
    Kr,81.0000,0.0000
    Kr,81.91348273,0.11593
    Kr,82.91412716,0.115
    Kr,83.9114977282,0.56987
    Kr,85.0000,0.0000
    Kr,85.9106106269,0.17279
    Rb,84.9117897379,0.7217
    Rb,86.0000,0.0000
    Rb,86.909180531,0.2783
    Sr,83.9134191,0.0056
    Sr,85.0000,0.0000
    Sr,85.9092606,0.0986
    Sr,86.9088775,0.07
    Sr,87.9056125,0.8258
    Y,88.9058403,1.0
    Zr,89.9046977,0.5145
    Zr,90.9056396,0.1122
    Zr,91.9050347,0.1715
    Zr,93.0000,0.0000
    Zr,93.9063108,0.1738
    Zr,95.0000,0.0000
    Zr,95.9082714,0.028
    Nb,92.906373,1.0
    Mo,91.90680796,0.1453
    Mo,93.0000,0.0000
    Mo,93.9050849,0.0915
    Mo,94.90583877,0.1584
    Mo,95.90467612,0.1667
    Mo,96.90601812,0.096
    Mo,97.90540482,0.2439
    Mo,99.0000,0.0000
    Mo,99.9074718,0.0982
    Ru,95.90759025,0.0554
    Ru,97.0000,0.0000
    Ru,97.9052868,0.0187
    Ru,98.9059341,0.1276
    Ru,99.9042143,0.126
    Ru,100.9055769,0.1706
    Ru,101.9043441,0.3155
    Ru,103.0000,0.0000
    Ru,103.9054275,0.1862
    Rh,102.905498,1.0
    Pd,101.9056022,0.0102
    Pd,103.0000,0.0000
    Pd,103.9040305,0.1114
    Pd,104.9050796,0.2233
    Pd,105.9034804,0.2733
    Pd,107.0000,0.0000
    Pd,107.9038916,0.2646
    Pd,109.0000,0.0000
    Pd,109.9051722,0.1172
    Ag,106.9050916,0.51839
    Ag,108.0000,0.0000
    Ag,108.9047553,0.48161
    Cd,105.9064599,0.0125
    Cd,106.0000,0.0000
    Cd,107.0000,0.0000
    Cd,107.9041834,0.0089
    Cd,108.0000,0.0000
    Cd,109.0000,0.0000
    Cd,109.90300661,0.1249
    Cd,110.90418287,0.128
    Cd,111.90276287,0.2413
    Cd,112.90440813,0.1222
    Cd,113.90336509,0.2873
    Cd,114.9000,0.0000
    Cd,115.90476315,0.0749
    In,112.90406184,0.0429
    In,114.0000,0.0000
    In,114.903878776,0.9571
    Sn,111.90482387,0.0097
    Sn,113.0000,0.0000
    Sn,113.9027827,0.0066
    Sn,114.903344699,0.0034
    Sn,115.9017428,0.1454
    Sn,116.90295398,0.0768
    Sn,117.90160657,0.2422
    Sn,118.90331117,0.0859
    Sn,119.90220163,0.3258
    Sn,121.0000,0.0000
    Sn,121.9034438,0.0463
    Sn,123.0000,0.0000
    Sn,123.9052766,0.0579
    Sb,120.903812,0.5721
    Sb,122.0000,0.0000
    Sb,122.9042132,0.4279
    Te,119.9040593,0.0009
    Te,121.0000,0.0000
    Te,121.9030435,0.0255
    Te,122.9042698,0.0089
    Te,123.9028171,0.0474
    Te,124.9044299,0.0707
    Te,125.9033109,0.1884
    Te,127.0000,0.0000
    Te,127.90446128,0.3174
    Te,129.0000,0.0000
    Te,129.906222748,0.3408
    I,126.9044719,1.0
    Xe,123.905892,0.000952
    Xe,125.0000,0.0000
    Xe,125.9042983,0.00089
    Xe,127.0000,0.0000
    Xe,127.903531,0.019102
    Xe,128.9047808611,0.264006
    Xe,129.903509349,0.04071
    Xe,130.90508406,0.212324
    Xe,131.9041550856,0.269086
    Xe,133.0000,0.0000
    Xe,133.90539466,0.104357
    Xe,135.0000,0.0000
    Xe,135.907214484,0.088573
    Cs,132.905451961,1.0
    Ba,129.9063207,0.00106
    Ba,130.9000,0.000
    Ba,131.9050611,0.00101
    Ba,132.9000,0.000
    Ba,133.90450818,0.02417
    Ba,134.90568838,0.06592
    Ba,135.90457573,0.07854
    Ba,136.90582714,0.11232
    Ba,137.905247,0.71698
    La,137.9071149,0.0008881
    La,138.9063563,0.9991119
    Ce,135.90712921,0.00185
    Ce,136.9000,0.0000
    Ce,137.905991,0.00251
    Ce,138.9000,0.0000
    Ce,139.9054431,0.8845
    Ce,140.9000,0.0000
    Ce,141.9092504,0.11114
    Pr,140.9076576,1.0
    Nd,141.907729,0.27152
    Nd,142.90982,0.12174
    Nd,143.910093,0.23798
    Nd,144.9125793,0.08293
    Nd,145.9131226,0.17189
    Nd,147.0000,0.0000
    Nd,147.9168993,0.05756
    Nd,149.0000,0.0000
    Nd,149.9209022,0.05638
    Sm,143.9120065,0.0307
    Sm,145.0000,0.0000
    Sm,146.0000,0.0000
    Sm,146.9149044,0.1499
    Sm,147.9148292,0.1124
    Sm,148.9171921,0.1382
    Sm,149.9172829,0.0738
    Sm,151.0000,0.0000
    Sm,151.9197397,0.2675
    Sm,153.0000,0.0000
    Sm,153.9222169,0.2275
    Eu,150.9198578,0.4781
    Eu,152.0000,0.0000
    Eu,152.921238,0.5219
    Gd,151.9197995,0.002
    Gd,153.0000,0.0000
    Gd,153.9208741,0.0218
    Gd,154.9226305,0.148
    Gd,155.9221312,0.2047
    Gd,156.9239686,0.1565
    Gd,157.9241123,0.2484
    Gd,159.0000,0.0000
    Gd,159.9270624,0.2186
    Tb,158.9253547,1.0
    Dy,155.9242847,0.00056
    Dy,157.0000, 0.0000
    Dy,157.9244159,0.00095
    Dy,159.0000, 0.0000
    Dy,159.9252046,0.02329
    Dy,160.9269405,0.18889
    Dy,161.9268056,0.25475
    Dy,162.9287383,0.24896
    Dy,163.9291819,0.2826
    Ho,164.9303288,1.0
    Er,161.9287884,0.00139
    Er,163.0000,0.0000
    Er,163.9292088,0.01601
    Er,165.0000,0.0000
    Er,165.9302995,0.33503
    Er,166.9320546,0.22869
    Er,167.9323767,0.26978
    Er,169.0000,0.0000
    Er,169.9354702,0.1491
    Tm,168.9342179,1.0
    Yb,167.9338896,0.00123
    Yb,169.0000,0.0000
    Yb,169.9347664,0.02982
    Yb,170.9363302,0.1409
    Yb,171.9363859,0.2168
    Yb,172.9382151,0.16103
    Yb,173.9388664,0.32026
    Yb,175.0000,0.0000
    Yb,175.9425764,0.12996
    Lu,174.9407752,0.97401
    Lu,175.9426897,0.02599
    Hf,173.9400461,0.0016
    Hf,175.0000,0.0000
    Hf,175.9414076,0.0526
    Hf,176.9432277,0.186
    Hf,177.9437058,0.2728
    Hf,178.9458232,0.1362
    Hf,179.946557,0.3508
    Ta,179.9474648,0.0001201
    Ta,180.9479958,0.9998799
    W,179.9467108,0.0012
    W,181.0000,0.0000
    W,181.94820394,0.265
    W,182.95022275,0.1431
    W,183.95093092,0.3064
    W,185.0000,0.0000
    W,185.9543628,0.2843
    Re,184.9529545,0.374
    Re,186.0000,0.0000
    Re,186.9557501,0.626
    Os,183.9524885,0.0002
    Os,185.0000,0.0000
    Os,185.953835,0.0159
    Os,186.9557474,0.0196
    Os,187.9558352,0.1324
    Os,188.9581442,0.1615
    Os,189.9584437,0.2626
    Os,191.0000,0.0000
    Os,191.961477,0.4078
    Ir,190.9605893,0.373
    Ir,192.0000,0.0000
    Ir,192.9629216,0.627
    Pt,189.9599297,0.00012
    Pt,191.0000,0.0000
    Pt,191.9610387,0.00782
    Pt,193.0000,0.0000
    Pt,193.9626809,0.3286
    Pt,194.9647917,0.3378
    Pt,195.96495209,0.2521
    Pt,197.0000,0.0000
    Pt,197.9678949,0.07356
    Au,196.96656879,1.0
    Hg,195.9658326,0.0015
    Hg,197.0000,0.0000
    Hg,197.9667686,0.0997
    Hg,198.96828064,0.1687
    Hg,199.96832659,0.231
    Hg,200.97030284,0.1318
    Hg,201.9706434,0.2986
    Hg,203.0000,0.0000
    Hg,203.97349398,0.0687
    Tl,202.9723446,0.2952
    Tl,204.0000,0.0000
    Tl,204.9744278,0.7048
    Pb,203.973044,0.014
    Pb,205.0000,0.0000
    Pb,205.9744657,0.241
    Pb,206.9758973,0.221
    Pb,207.9766525,0.524
    Bi,208.9803991,1.0
    Th,232.0380558,1.0
    Pa,231.0358842,1.0
    U,234.0409523,5.4e-05
    U,235.0439301,0.007204
    U,236.0000,0.0000
    U,237.0000,0.0000
    U,238.0507884,0.992742
"""

def get_isotope_data(data_path=None):
    """
    This function parses either the ISOTOPE_RAW_DATA above or the provided
    csv file provided by data_path and returns the data properly formatted
    for use by isocor.

    The input must be csv formatted with element, isotope exact mass, and
    abundance. Due to isocor, there cannot be gaps in the masses. 


    Args:
        data_path (str, optional): alterantive path to isotope data. 
        Defaults to None.
    """

    def _stripColNames(df):
        df.rename(columns=lambda x: x.strip())

    def _stripCol(df, listcolumns):
        for col in listcolumns:
            df[col] = df[col].str.strip()

    def _makeIsotopesDict(df):
        gb = df.groupby('element')
        d = {}
        for g in gb.groups:
            dg = gb.get_group(g).sort_values(by='mass').to_dict('list')
            e = dg.pop('element')[0]
            d[e] = dg
        return d
    
    if data_path is None:
        parsed_data = []
        iso_data = ISOTOPE_RAW_DATA[0].strip()
        for line in iso_data[1:]:
            element, mass, abundance = line.strip().split(",")
            dfIsotopes.append({
                "element": element,
                "mass": Decimal(mass),
                "abundance": np.float64(abundance)
            })
        dfIsotopes = pd.DataFrame(parsed_data)
    else:
        with open(str(data_path), 'r', encoding='utf-8') as fp:
            dfIsotopes = pd.read_csv(fp, converters={'mass': Decimal, 'abundance': np.float64})
    _stripColNames(dfIsotopes)
    _stripCol(dfIsotopes, ['element', ])
    dictIsotopes = _makeIsotopesDict(dfIsotopes)
    return dictIsotopes

==> khipu_metabolomics-2.0.2-py3-none-any.whl: khipu/isotopes.py <==
# this contains the raw isotope data for the isotope correction portion of khipu

from decimal import Decimal
import pandas as pd
import numpy as np

# this is a master list of isotopes sourced from NIST
# update this if the NAPs in yours system are non-canonical
# see the function below on how to override this list.

ISOTOPE_RAW_DATA = """element,mass,abundance
    H,1.00782503223,0.999885
    H,2.01410177812,0.000115
    He,3.0160293201,1.34e-06
    He,4.00260325413,0.99999866
    Li,6.0151228874,0.0759
    Li,7.0160034366,0.9241
    Be,9.012183065,1.0
    B,10.01293695,0.199
    B,11.00930536,0.801
    C,12.0,0.9893
    C,13.00335483507,0.0107
    N,14.00307400443,0.99636
    N,15.00010889888,0.00364
    O,15.99491461957,0.99757
    O,16.9991317565,0.00038
    O,17.99915961286,0.00205
    F,18.99840316273,1.0
    Ne,19.9924401762,0.9048
    Ne,20.993846685,0.0027
    Ne,21.991385114,0.0925
    Na,22.989769282,1.0
    Mg,23.985041697,0.7899
    Mg,24.985836976,0.1
    Mg,25.982592968,0.1101
    Al,26.98153853,1.0
    Si,27.97692653465,0.92223
    Si,28.9764946649,0.04685
    Si,29.973770136,0.03092
    P,30.97376199842,1.0
    S,31.9720711744,0.9499
    S,32.9714589098,0.0075
    S,33.967867004,0.0425
    S,35.0000,0.0000
    S,35.96708071,0.0001
    Cl,34.968852682,0.7576
    Cl,36.0000,0.0000
    Cl,36.965902602,0.2424
    Ar,35.967545105,0.003336
    Ar,37.00000,0.0000
    Ar,37.96273211,0.000629
    Ar,39.00000,0.0000
    Ar,39.9623831237,0.996035
    K,38.9637064864,0.932581
    K,39.963998166,0.000117
    K,40.9618252579,0.067302
    Ca,39.962590863,0.96941
    Ca,41.0000,0.0000
    Ca,41.95861783,0.00647
    Ca,42.95876644,0.00135
    Ca,43.95548156,0.02086
    Ca,45.0000,0.0000
    Ca,45.953689,4e-05
    Ca,46.9000,0.0000
    Ca,47.95252276,0.00187
    Sc,44.95590828,1.0
    Ti,45.95262772,0.0825
    Ti,46.95175879,0.0744
    Ti,47.94794198,0.7372
    Ti,48.94786568,0.0541
    Ti,49.94478689,0.0518
    V,49.94715601,0.0025
    V,50.94395704,0.9975
    Cr,49.94604183,0.04345
    Cr,51.0000,0.0000
    Cr,51.94050623,0.83789
    Cr,52.94064815,0.09501
    Cr,53.93887916,0.02365
    Mn,54.93804391,1.0
    Fe,53.93960899,0.05845
    Fe,55.0000,0.0000
    Fe,55.93493633,0.91754
    Fe,56.93539284,0.02119
    Fe,57.93327443,0.00282
    Co,58.93319429,1.0
    Ni,57.93534241,0.68077
    Ni,59.0000,0.0000
    Ni,59.93078588,0.26223
    Ni,60.93105557,0.011399
    Ni,61.92834537,0.036346
    Ni,63.0000,0.0000
    Ni,63.92796682,0.009255
    Cu,62.92959772,0.6915
    Cu,64.0000,0.0000
    Cu,64.9277897,0.3085
    Zn,63.92914201,0.4917
    Zn,65.0000,0.0000
    Zn,65.92603381,0.2773
    Zn,66.92712775,0.0404
    Zn,67.92484455,0.1845
    Zn,69.0000,0.0000
    Zn,69.9253192,0.0061
    Ga,68.9255735,0.60108
    Ga,70.0000,0.0000
    Ga,70.92470258,0.39892
    Ge,69.92424875,0.2057
    Ge,71.0000,0.0000
    Ge,71.922075826,0.2745
    Ge,72.923458956,0.0775
    Ge,73.921177761,0.365
    Ge,75.0000,0.0000
    Ge,75.921402726,0.0773
    As,74.92159457,1.0
    Se,73.922475934,0.0089
    Se,75.0000,0.0000
    Se,75.919213704,0.0937
    Se,76.919914154,0.0763
    Se,77.91730928,0.2377
    Se,79.0000,0.0000
    Se,79.9165218,0.4961
    Se,81.0000,0.0000
    Se,81.9166995,0.0873
    Br,78.9183376,0.5069
    Br,80.0000,0.0000
    Br,80.9162897,0.4931
    Kr,77.92036494,0.00355
    Kr,79.0000,0.0000
    Kr,79.91637808,0.02286
    Kr,81.0000,0.0000
    Kr,81.91348273,0.11593
    Kr,82.91412716,0.115
    Kr,83.9114977282,0.56987
    Kr,85.0000,0.0000
    Kr,85.9106106269,0.17279
    Rb,84.9117897379,0.7217
    Rb,86.0000,0.0000
    Rb,86.909180531,0.2783
    Sr,83.9134191,0.0056
    Sr,85.0000,0.0000
    Sr,85.9092606,0.0986
    Sr,86.9088775,0.07
    Sr,87.9056125,0.8258
    Y,88.9058403,1.0
    Zr,89.9046977,0.5145
    Zr,90.9056396,0.1122
    Zr,91.9050347,0.1715
    Zr,93.0000,0.0000
    Zr,93.9063108,0.1738
    Zr,95.0000,0.0000
    Zr,95.9082714,0.028
    Nb,92.906373,1.0
    Mo,91.90680796,0.1453
    Mo,93.0000,0.0000
    Mo,93.9050849,0.0915
    Mo,94.90583877,0.1584
    Mo,95.90467612,0.1667
    Mo,96.90601812,0.096
    Mo,97.90540482,0.2439
    Mo,99.0000,0.0000
    Mo,99.9074718,0.0982
    Ru,95.90759025,0.0554
    Ru,97.0000,0.0000
    Ru,97.9052868,0.0187
    Ru,98.9059341,0.1276
    Ru,99.9042143,0.126
    Ru,100.9055769,0.1706
    Ru,101.9043441,0.3155
    Ru,103.0000,0.0000
    Ru,103.9054275,0.1862
    Rh,102.905498,1.0
    Pd,101.9056022,0.0102
    Pd,103.0000,0.0000
    Pd,103.9040305,0.1114
    Pd,104.9050796,0.2233
    Pd,105.9034804,0.2733
    Pd,107.0000,0.0000
    Pd,107.9038916,0.2646
    Pd,109.0000,0.0000
    Pd,109.9051722,0.1172
    Ag,106.9050916,0.51839
    Ag,108.0000,0.0000
    Ag,108.9047553,0.48161
    Cd,105.9064599,0.0125
    Cd,106.0000,0.0000
    Cd,107.0000,0.0000
    Cd,107.9041834,0.0089
    Cd,108.0000,0.0000
    Cd,109.0000,0.0000
    Cd,109.90300661,0.1249
    Cd,110.90418287,0.128
    Cd,111.90276287,0.2413
    Cd,112.90440813,0.1222
    Cd,113.90336509,0.2873
    Cd,114.9000,0.0000
    Cd,115.90476315,0.0749
    In,112.90406184,0.0429
    In,114.0000,0.0000
    In,114.903878776,0.9571
    Sn,111.90482387,0.0097
    Sn,113.0000,0.0000
    Sn,113.9027827,0.0066
    Sn,114.903344699,0.0034
    Sn,115.9017428,0.1454
    Sn,116.90295398,0.0768
    Sn,117.90160657,0.2422
    Sn,118.90331117,0.0859
    Sn,119.90220163,0.3258
    Sn,121.0000,0.0000
    Sn,121.9034438,0.0463
    Sn,123.0000,0.0000
    Sn,123.9052766,0.0579
    Sb,120.903812,0.5721
    Sb,122.0000,0.0000
    Sb,122.9042132,0.4279
    Te,119.9040593,0.0009
    Te,121.0000,0.0000
    Te,121.9030435,0.0255
    Te,122.9042698,0.0089
    Te,123.9028171,0.0474
    Te,124.9044299,0.0707
    Te,125.9033109,0.1884
    Te,127.0000,0.0000
    Te,127.90446128,0.3174
    Te,129.0000,0.0000
    Te,129.906222748,0.3408
    I,126.9044719,1.0
    Xe,123.905892,0.000952
    Xe,125.0000,0.0000
    Xe,125.9042983,0.00089
    Xe,127.0000,0.0000
    Xe,127.903531,0.019102
    Xe,128.9047808611,0.264006
    Xe,129.903509349,0.04071
    Xe,130.90508406,0.212324
    Xe,131.9041550856,0.269086
    Xe,133.0000,0.0000
    Xe,133.90539466,0.104357
    Xe,135.0000,0.0000
    Xe,135.907214484,0.088573
    Cs,132.905451961,1.0
    Ba,129.9063207,0.00106
    Ba,130.9000,0.000
    Ba,131.9050611,0.00101
    Ba,132.9000,0.000
    Ba,133.90450818,0.02417
    Ba,134.90568838,0.06592
    Ba,135.90457573,0.07854
    Ba,136.90582714,0.11232
    Ba,137.905247,0.71698
    La,137.9071149,0.0008881
    La,138.9063563,0.9991119
    Ce,135.90712921,0.00185
    Ce,136.9000,0.0000
    Ce,137.905991,0.00251
    Ce,138.9000,0.0000
    Ce,139.9054431,0.8845
    Ce,140.9000,0.0000
    Ce,141.9092504,0.11114
    Pr,140.9076576,1.0
    Nd,141.907729,0.27152
    Nd,142.90982,0.12174
    Nd,143.910093,0.23798
    Nd,144.9125793,0.08293
    Nd,145.9131226,0.17189
    Nd,147.0000,0.0000
    Nd,147.9168993,0.05756
    Nd,149.0000,0.0000
    Nd,149.9209022,0.05638
    Sm,143.9120065,0.0307
    Sm,145.0000,0.0000
    Sm,146.0000,0.0000
    Sm,146.9149044,0.1499
    Sm,147.9148292,0.1124
    Sm,148.9171921,0.1382
    Sm,149.9172829,0.0738
    Sm,151.0000,0.0000
    Sm,151.9197397,0.2675
    Sm,153.0000,0.0000
    Sm,153.9222169,0.2275
    Eu,150.9198578,0.4781
    Eu,152.0000,0.0000
    Eu,152.921238,0.5219
    Gd,151.9197995,0.002
    Gd,153.0000,0.0000
    Gd,153.9208741,0.0218
    Gd,154.9226305,0.148
    Gd,155.9221312,0.2047
    Gd,156.9239686,0.1565
    Gd,157.9241123,0.2484
    Gd,159.0000,0.0000
    Gd,159.9270624,0.2186
    Tb,158.9253547,1.0
    Dy,155.9242847,0.00056
    Dy,157.0000, 0.0000
    Dy,157.9244159,0.00095
    Dy,159.0000, 0.0000
    Dy,159.9252046,0.02329
    Dy,160.9269405,0.18889
    Dy,161.9268056,0.25475
    Dy,162.9287383,0.24896
    Dy,163.9291819,0.2826
    Ho,164.9303288,1.0
    Er,161.9287884,0.00139
    Er,163.0000,0.0000
    Er,163.9292088,0.01601
    Er,165.0000,0.0000
    Er,165.9302995,0.33503
    Er,166.9320546,0.22869
    Er,167.9323767,0.26978
    Er,169.0000,0.0000
    Er,169.9354702,0.1491
    Tm,168.9342179,1.0
    Yb,167.9338896,0.00123
    Yb,169.0000,0.0000
    Yb,169.9347664,0.02982
    Yb,170.9363302,0.1409
    Yb,171.9363859,0.2168
    Yb,172.9382151,0.16103
    Yb,173.9388664,0.32026
    Yb,175.0000,0.0000
    Yb,175.9425764,0.12996
    Lu,174.9407752,0.97401
    Lu,175.9426897,0.02599
    Hf,173.9400461,0.0016
    Hf,175.0000,0.0000
    Hf,175.9414076,0.0526
    Hf,176.9432277,0.186
    Hf,177.9437058,0.2728
    Hf,178.9458232,0.1362
    Hf,179.946557,0.3508
    Ta,179.9474648,0.0001201
    Ta,180.9479958,0.9998799
    W,179.9467108,0.0012
    W,181.0000,0.0000
    W,181.94820394,0.265
    W,182.95022275,0.1431
    W,183.95093092,0.3064
    W,185.0000,0.0000
    W,185.9543628,0.2843
    Re,184.9529545,0.374
    Re,186.0000,0.0000
    Re,186.9557501,0.626
    Os,183.9524885,0.0002
    Os,185.0000,0.0000
    Os,185.953835,0.0159
    Os,186.9557474,0.0196
    Os,187.9558352,0.1324
    Os,188.9581442,0.1615
    Os,189.9584437,0.2626
    Os,191.0000,0.0000
    Os,191.961477,0.4078
    Ir,190.9605893,0.373
    Ir,192.0000,0.0000
    Ir,192.9629216,0.627
    Pt,189.9599297,0.00012
    Pt,191.0000,0.0000
    Pt,191.9610387,0.00782
    Pt,193.0000,0.0000
    Pt,193.9626809,0.3286
    Pt,194.9647917,0.3378
    Pt,195.96495209,0.2521
    Pt,197.0000,0.0000
    Pt,197.9678949,0.07356
    Au,196.96656879,1.0
    Hg,195.9658326,0.0015
    Hg,197.0000,0.0000
    Hg,197.9667686,0.0997
    Hg,198.96828064,0.1687
    Hg,199.96832659,0.231
    Hg,200.97030284,0.1318
    Hg,201.9706434,0.2986
    Hg,203.0000,0.0000
    Hg,203.97349398,0.0687
    Tl,202.9723446,0.2952
    Tl,204.0000,0.0000
    Tl,204.9744278,0.7048
    Pb,203.973044,0.014
    Pb,205.0000,0.0000
    Pb,205.9744657,0.241
    Pb,206.9758973,0.221
    Pb,207.9766525,0.524
    Bi,208.9803991,1.0
    Th,232.0380558,1.0
    Pa,231.0358842,1.0
    U,234.0409523,5.4e-05
    U,235.0439301,0.007204
    U,236.0000,0.0000
    U,237.0000,0.0000
    U,238.0507884,0.992742"""

def get_isotope_data(data_path=None):
    """
    This function parses either the ISOTOPE_RAW_DATA above or the provided
    csv file provided by data_path and returns the data properly formatted
    for use by isocor.

    The input must be csv formatted with element, isotope exact mass, and
    abundance. Due to isocor, there cannot be gaps in the masses. 


    Args:
        data_path (str, optional): alterantive path to isotope data. 
        Defaults to None.
    """

    def _stripColNames(df):
        df.rename(columns=lambda x: x.strip())

    def _stripCol(df, listcolumns):
        for col in listcolumns:
            df[col] = df[col].str.strip()

    def _makeIsotopesDict(df):
        gb = df.groupby('element')
        d = {}
        for g in gb.groups:
            dg = gb.get_group(g).sort_values(by='mass').to_dict('list')
            e = dg.pop('element')[0]
            d[e] = dg
        return d
    
    if data_path is None:
        parsed_data = []
        iso_data = ISOTOPE_RAW_DATA[0].split("\n")
        for line in iso_data[1:]:
            element, mass, abundance = line.strip().split(",")
            parsed_data.append({
                "element": element,
                "mass": Decimal(mass),
                "abundance": np.float64(abundance)
            })
        dfIsotopes = pd.DataFrame(parsed_data)
    else:
        with open(str(data_path), 'r', encoding='utf-8') as fp:
            dfIsotopes = pd.read_csv(fp, converters={'mass': Decimal, 'abundance': np.float64})
    _stripColNames(dfIsotopes)
    _stripCol(dfIsotopes, ['element', ])
    dictIsotopes = _makeIsotopesDict(dfIsotopes)
    return dictIsotopes
